;************************************************************************
;
; TITLE: 	Programmable Peripheral Controller
;
; CPU: 		C8051F334
; TARGET:       
; DESC:         I2C support interface to IO MODULE
;
;************************************************************************
	cpu 8051
	listing off

	if 0

I2C READ
STATUS[1] IO[1] TEMPERATURE[2] AN0[2] AN1[2] AN2[2] AN3[2]

I2C WRITE
IOCMD[1] IODAT[1] IODIR[1]

COMMANDS:
00: WRITE DATA TO OUTPUT PORT and maybe DIR
01: WRITE DATA TO DIR PORT
02..0F: set read pointer
SMBSTAT		DS 1
SMBINPS		DS 1
SMBANAL		DS 4
		DS 10

10: READ VERSION - the next read will read out the TITLE
11: READ FLASH from addres[2] - the next read will start reading bytes continuously until STOP
12: BRIDGE MONITOR OFF
13: BRIDGE MONITOR ON
	monitors the voltage levels on the output and overrides on fault

	endif
	relaxed on
	page 0
   include f330AS.inc                  ; Include register definition file.
	listing on

;***************************** CONSTANTS **********************************

SDA	bit P0.0
SCL	bit P0.1
SDA2	bit P2.0

AN0	bit P0.2	;PIN 0 via 100K+10K
AN1	bit P0.3        ;PIN 1  "
AN2	bit P0.6        ;PIN 2  "
AN3	bit P0.7        ;PIN 3  "
RST	bit P0.4        ;Tied to HOST reset line - dr
RXD	bit P0.5

; MODULE I/O
; These pins go directly to the MODULE and are coupled to the
; Propeller I/O by 10K resistors so that this CPU can override
; the Propeller's control as necessary such as fault conditions
;
M0	bit P1.0	; IO0 and PIN 0 digital input
M1	bit P1.1	; IO1
M2	bit P1.2	; IO2 and PIN 1 digital input
M3	bit P1.3	; IO3
M4	bit P1.4	; IO4 and PIN 2 digital input
M5	bit P1.5	; IO5
M6	bit P1.6	; IO6 and PIN 3 digital input
M7	bit P1.7	; IO7


SYSTEMCLOCK	= 24500000           ; SYSCLK frequency in Hz
BAUDRATE	= 9600           ; Baud rate of UART in bps


;***************************** VARIABLES **********************************

	SEGMENT bitdata
	org 20h

	SEGMENT data
AMUXCH		DS 1
cold		DS 1



movw	macro	dst,src
	mov	dst,src&$ff
	mov	dst+1,src>>8
	endm


;-----------------------------------------------------------------------------
; RESET and INTERRUPT VECTORS
;-----------------------------------------------------------------------------
	SEGMENT code
            ; Reset Vector
	ORG 00h
	ljmp 	Reset                     ; reset vector.

	org 03h
VINT0:
	mov	A,#1
	jmp	TRAP

	org 0bh
VTMR0:	mov	A,#2
	jmp	TRAP

	org 13h
VINT1:	mov	A,#3
	jmp	TRAP

	org 1bh
VTMR1:	mov	A,#4
	jmp	TRAP

	org 23h
VUART0:	mov	A,#5
	jmp	TRAP

	org 2bh
VTMR2:	jmp	Timer2_ISR

	org 33h
VSPI0:	mov	A,#7
	jmp	TRAP

	org 3bh
VSMB0: 	jmp	SMBus_ISR

	org 43h
V8:	mov	A,#9
	jmp	TRAP

	org 4bh
VADCW0:	mov	A,#10
	jmp	TRAP

	org 53h
VADC0:	jmp	ADC0_ISR

	org 5bh
VPCA:	mov	A,#12
	jmp	TRAP

	org 63h
VCOM0:	mov	A,#13
	jmp	TRAP

	org 6bh
V13:	mov	A,#14
	jmp	TRAP


	org 73h
VTMR3:  jmp	Timer3_ISR


;-----------------------------------------------------------------------------
; CODE SEGMENT
;-----------------------------------------------------------------------------


 	org 80h
SLAVEADR:
	db 0F2h		;default i2c address - patch the file directly to change

TITLE:
	db	"PPC I/O V1.0A "
	db	"Analog and digital I/O "
	db	" (BUILD: ",DATE,"-"
BUILD:	db	TIME,")",0

TRAP:   mov	smbstat,A
Reset:
	mov	SP,#0EFh
            ; Disable the WDT.
	anl	PCA0MD, #~(040h)      ; clear Watchdog Enable bit

	orl	OSCICN, #3		; full speed
	mov	CLKSEL,#0
	mov	RSTSRC,#0x04		; Enable missing clock detector

            				; Enable the Port I/O Crossbar
	orl	P0SKIP,#0FCh
	mov	P0MDIN,#033h
	anl	P0MDOUT,#0FCh

	orl	P1SKIP, #0FFh
	orl	P1MDIN,#0FFh		; no analog inputs
	orl	P1MDOUT,#0FFh		; pull pin drivers low (for startup safety)
	mov	P1,#0

	mov	XBR0,#0
	mov	XBR1, #40h              ; enable Crossbar

	call	SMBBUS
	setb	EA			; Global interrupt enable

	call	Timer2_Init
	call    ADC0_Init

MAIN:
	mov	smbstat,smbflg
	mov	smbstat+1,P1
	jnb	smbready,MAIN
	call	PROCESS_SMB
	clr	smbready
	mov	smbcmd,#0
	jmp	MAIN



PROCESS_SMB:
	mov	A,smbcmd
	orl	A,#0
	jnz	PROC1
	ret
PROC1:  cjne	A,#1,PROC2
	mov     P1,smbdat 		; WRITE PORT
	mov	P1MDOUT,smbdat+1        ; WRITE DIR
	ret
PROC2:  cjne	A,#2,PROC2A		; write P1 DIR only
	mov	P1MDOUT,smbdat
	ret
PROC2A: cjne	A,#3,PROC3		; unlock
	mov	P1MDOUT,#0              ; no push-pull
	mov	P1,#0FFh		; release open-drain as well
	ret


PROC3:  cjne	A,#0FDh,PROC4 		;read version
	mov	DPL,#TITLE&0FFh
	mov	DPH,#TITLE/256
	jmp	RDFL
PROC4:  cjne	A,#0FEh,PROC5 		;read flash from address
	mov	DPL,smbdat
	mov	DPH,smbdat+1
RDFL:	mov	r0,#smbtxbuf+SMBTXSZ	; use aux buffer at end of normal tx buffer
	mov	r1,#SMBTXSZ2
rdflp:	clr	A
	movc	A,@A+DPTR
	mov	@r0,A
	inc	DPL
	inc	r0
	djnz	r1,rdflp
	mov	smbTxRd,#SMBTXSZ	; point past end of normal buffer
	ret
PROC5:  cjne	A,#0FFh,PROC6
	jmp	Reset
PROC6:	mov	smbTxRd,smbdat		; set read pointer
	ret




;******************************* ADC *****************************
;
; Read 2,3,6,7, and 16 (temp)
;
;


ADC0_Init:
	mov	AMUXCH,#0
	mov	ADC0CN,#02h		;ADC0 disabled, normal tracking,
                                       ; conversion triggered on TMR2 overflow

	mov	REF0CN,#0Eh		; Vref = VDD, TEMP SENSOR ON
	mov	AMX0P,#0		;ADC0 initial positive input = P0.1
	mov	AMX0N,#11h		; ADC0 negative input = GND
                                        ;i.e., single ended mode

	mov	ADC0CF,#((SYSCLK/3000000)-1)<<3	;Set SAR clock to 3MHz  - right justify
	orl	EIE1,#08h		; Enable ADC0 EOC interrupt
	setb	AD0EN			; Enable ADC0
	ret


Timer2_Init:
	mov	TMR2CN,#0		; Stop Timer2; Clear TF2;
                                        ; use SYSCLK as timebase, 16-bit
                                        ; auto-reload
	orl	CKCON,#010h		; Select SYSCLK for timer 2 source
TMR2VAL = 0FFFFh- (SYSCLK / 10000)
	mov	TMR2RLH,#(TMR2VAL/256) ; Init reload value for 10 us
	mov	TMR2RLL,#(TMR2VAL&0FFh)
	mov	TMR2H,#0ffh		; Set to reload immediately
	mov	TMR2L,#0ffh
	setb	ET2			; Enable Timer2 interrupts
	setb	TR2			; Start Timer2
	ret

; TIMER 2 INTERRUPT SERVIDE ROUTINE
; SELECT NEXT ADC CHANNEL FROM LOOKUP TABLE
;

Timer2_ISR:
	PUSH    ACC
	PUSH    PSW
	MOV     PSW,#00H
	PUSH    0
	clr	TF2H			; Clear Timer2 interrupt flag
TMX1:	mov	A,AMUXCH
TMX2:	add	A,#1			; table offset
	call	AMUXTBL			; lookup next channel to scan
	orl	A,#0                    ; end of lookup table?
	jnz	TMX3
	mov	AMUXCH,#0               ; reset to beginning of table
	jmp	TMX1 			; reread from start of table
TMX3:	mov	AMX0P,A                 ; select the next ADC channel
;	inc	AMUXCH
	pop	0
	pop	PSW
	pop	ACC
	reti

AMUXTBL:
	movc	A,@A+PC
	ret
	db      2,3,6,7,16,0,0

; ANALOG CONVERTER INTERRUPT SERVICE ROUTINE
;
ADC0_ISR:
	PUSH    ACC
	PUSH    PSW
	MOV     PSW,#00H
	PUSH    0
	clr	AD0INT
	mov	A,AMUXCH	;write result to latches
	anl	A,#07		; security
	rl	A               ; 2 bytes/sample
	add	A,#adcbuf
	mov	r0,A
	mov	@r0,ADC0L
	inc	r0
	mov	@r0,ADC0H
	inc	AMUXCH
       	pop	0
	pop	PSW
	pop	ACC
	reti



;************************************ I2C ************************
;
; Read of slave always returns all registers in this order
; READ: STATUS[1],INPUTS[1], ANALOG[4]
; WRITE: CMD, DATA

;SLAVE_ADDR	= 0xF2		; Device addresses (7 bits, lsb is a don't care)
;
;

	include I2CSLAVE.i51

smbcmd		= smbrxbuf
smbdat		= smbrxbuf+1

smbstat		= smbtxbuf
adcbuf		= smbtxbuf+2



;-----------------------------------------------------------------------------
; End of file.

END
